@@ -6,7 +6,7 @@
  * handling
  *
  * (c) NLnet Labs, 2004-2006
- * 
+ *
  * See the file LICENSE for the license
  */
 
@@ -23,16 +23,22 @@
 ldns_lookup_table ldns_signing_algorithms[] = {
         { LDNS_SIGN_RSAMD5, "RSAMD5" },
         { LDNS_SIGN_RSASHA1, "RSASHA1" },
+        { LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1_NSEC3" },
 #ifdef USE_SHA2
         { LDNS_SIGN_RSASHA256, "RSASHA256" },
+        { LDNS_SIGN_RSASHA256_NSEC3, "RSASHA256_NSEC3" },
         { LDNS_SIGN_RSASHA512, "RSASHA512" },
+        { LDNS_SIGN_RSASHA512_NSEC3, "RSASHA512_NSEC3" },
 #endif
         { LDNS_SIGN_DSA, "DSA" },
+        { LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
         { LDNS_SIGN_HMACMD5, "hmac-md5.sig-alg.reg.int" },
+        { LDNS_SIGN_HMACSHA1, "hmac-sha1" },
+        { LDNS_SIGN_HMACSHA256, "hmac-sha256" },
         { 0, NULL }
 };
 
-#ifdef HAVE_SSL 
+#ifdef HAVE_SSL
 ldns_key_list *
 ldns_key_list_new()
 {
@@ -69,7 +75,7 @@ ldns_key_new()
 	return NULL;
 }
 
-ldns_status 
+ldns_status
 ldns_key_new_frm_fp(ldns_key **k, FILE *fp)
 {
 	return ldns_key_new_frm_fp_l(k, fp, NULL);
@@ -80,7 +86,7 @@ ldns_status
 ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm alg)
 {
 	ldns_key *k;
-	
+
 	k = ldns_key_new();
 	k->_key.key = ENGINE_load_private_key(e, key_id, UI_OpenSSL(), NULL);
 	ldns_key_set_algorithm(k, alg);
@@ -111,9 +117,9 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	if (!k || !d) {
 		return LDNS_STATUS_MEM_ERR;
 	}
-	
+
 	alg = 0;
-	
+
 	/* the file is highly structured. Do this in sequence */
 	/* RSA:
 	 * Private-key-format: v1.2
@@ -132,7 +138,7 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 
 	/* get the algorithm type, our file function strip ( ) so there are
 	 * not in the return string! */
-	if (ldns_fget_keyword_data_l(fp, "Algorithm", ": ", d, "\n", 
+	if (ldns_fget_keyword_data_l(fp, "Algorithm", ": ", d, "\n",
 				LDNS_MAX_LINELEN, line_nr) == -1) {
 		/* no alg information */
 		return LDNS_STATUS_SYNTAX_ALG_ERR;
@@ -142,21 +148,21 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		alg = LDNS_SIGN_RSAMD5;
 	}
 	if (strncmp(d, "2 DH", 2) == 0) {
-		alg = LDNS_DH; 
+		alg = LDNS_DH;
 	}
 	if (strncmp(d, "3 DSA", 2) == 0) {
-		alg = LDNS_SIGN_DSA; 
+		alg = LDNS_SIGN_DSA;
 	}
 	if (strncmp(d, "4 ECC", 2) == 0) {
-		alg = LDNS_ECC; 
+		alg = LDNS_ECC;
 	}
 	if (strncmp(d, "5 RSASHA1", 2) == 0) {
 		alg = LDNS_SIGN_RSASHA1;
 	}
-	if (strncmp(d, "6 DSA", 4) == 0) {
-		alg = LDNS_DSA_NSEC3; 
+	if (strncmp(d, "6 DSA", 2) == 0) {
+		alg = LDNS_DSA_NSEC3;
 	}
-	if (strncmp(d, "7 RSASHA1", 4) == 0) {
+	if (strncmp(d, "7 RSASHA1", 2) == 0) {
 		alg = LDNS_RSASHA1_NSEC3;
 	}
 
@@ -168,17 +174,39 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		fprintf(stderr, "version of ldns\n");
 #endif
 	}
-	if (strncmp(d, "9 RSASHA512", 2) == 0) {
+	if (strncmp(d, "9 RSASHA256", 2) == 0) {
+#ifdef USE_SHA2
+		alg = LDNS_SIGN_RSASHA256_NSEC3;
+#else
+		fprintf(stderr, "Warning: SHA256 not compiled into this ");
+		fprintf(stderr, "version of ldns\n");
+#endif
+	}
+	if (strncmp(d, "10 RSASHA512", 3) == 0) {
 #ifdef USE_SHA2
 		alg = LDNS_SIGN_RSASHA512;
 #else
 		fprintf(stderr, "Warning: SHA256 not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
+#endif
+	}
+	if (strncmp(d, "11 RSASHA512", 3) == 0) {
+#ifdef USE_SHA2
+		alg = LDNS_SIGN_RSASHA512_NSEC3;
+#else
+		fprintf(stderr, "Warning: SHA256 not compiled into this ");
+		fprintf(stderr, "version of ldns\n");
 #endif
 	}
 	if (strncmp(d, "157 HMAC-MD5", 4) == 0) {
 		alg = LDNS_SIGN_HMACMD5;
 	}
+	if (strncmp(d, "158 HMAC-SHA1", 4) == 0) {
+		alg = LDNS_SIGN_HMACSHA1;
+	}
+	if (strncmp(d, "159 HMAC-SHA256", 4) == 0) {
+		alg = LDNS_SIGN_HMACSHA256;
+	}
 
 	LDNS_FREE(d);
 
@@ -188,7 +216,9 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		case LDNS_RSASHA1_NSEC3:
 #ifdef USE_SHA2
 		case LDNS_SIGN_RSASHA256:
+		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
+		case LDNS_SIGN_RSASHA512_NSEC3:
 #endif
 			ldns_key_set_algorithm(k, alg);
 			rsa = ldns_key_new_frm_fp_rsa_l(fp, line_nr);
@@ -203,6 +233,8 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 			DSA_free(dsa);
 			break;
 		case LDNS_SIGN_HMACMD5:
+		case LDNS_SIGN_HMACSHA1:
+		case LDNS_SIGN_HMACSHA256:
 			ldns_key_set_algorithm(k, alg);
 			hmac = ldns_key_new_frm_fp_hmac_l(fp, line_nr, &hmac_size);
 			ldns_key_set_hmac_size(k, hmac_size);
@@ -235,14 +267,14 @@ RSA *
 ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 {
 	/* we parse
- 	 * Modulus: 
- 	 * PublicExponent: 
- 	 * PrivateExponent: 
- 	 * Prime1: 
- 	 * Prime2: 
- 	 * Exponent1: 
- 	 * Exponent2: 
- 	 * Coefficient: 
+ 	 * Modulus:
+ 	 * PublicExponent:
+ 	 * PrivateExponent:
+ 	 * Prime1:
+ 	 * Prime2:
+ 	 * Exponent1:
+ 	 * Exponent2:
+ 	 * Coefficient:
 	 *
 	 * man 3 RSA:
 	 *
@@ -272,14 +304,14 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	}
 
 	/* I could use functions again, but that seems an overkill,
-	 * allthough this also looks tedious 
+	 * allthough this also looks tedious
 	 */
 
 	/* Modules, rsa->n */
 	if (ldns_fget_keyword_data_l(f, "Modulus", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->n = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->n) {
 		goto error;
@@ -289,7 +321,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "PublicExponent", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->e = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->e) {
 		goto error;
@@ -299,7 +331,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "PrivateExponent", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->d = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->d) {
 		goto error;
@@ -309,17 +341,17 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Prime1", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->p = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->p) {
 		goto error;
 	}
-	
+
 	/* Prime2, rsa->q */
 	if (ldns_fget_keyword_data_l(f, "Prime2", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->q = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->q) {
 		goto error;
@@ -329,17 +361,17 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Exponent1", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->dmp1 = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->dmp1) {
 		goto error;
 	}
-	
+
 	/* Exponent2, rsa->dmq1 */
 	if (ldns_fget_keyword_data_l(f, "Exponent2", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->dmq1 = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->dmq1) {
 		goto error;
@@ -349,12 +381,12 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Coefficient", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	rsa->iqmp = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->iqmp) {
 		goto error;
 	}
-	
+
 	LDNS_FREE(buf);
 	LDNS_FREE(d);
 	return rsa;
@@ -394,7 +426,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Primep", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	dsa->p = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->p) {
 		goto error;
@@ -404,7 +436,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Subprimeq", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	dsa->q = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->q) {
 		goto error;
@@ -414,7 +446,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Baseg", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	dsa->g = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->g) {
 		goto error;
@@ -424,7 +456,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Private_valuex", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	dsa->priv_key = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->priv_key) {
 		goto error;
@@ -434,7 +466,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (ldns_fget_keyword_data_l(f, "Public_valuey", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 	dsa->pub_key = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->pub_key) {
 		goto error;
@@ -468,15 +500,15 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 
 	d = LDNS_XMALLOC(char, LDNS_MAX_LINELEN);
 	buf = LDNS_XMALLOC(unsigned char, LDNS_MAX_LINELEN);
-	
+
 	if (ldns_fget_keyword_data_l(f, "Key", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = b64_pton((const char*)d, buf, b64_ntop_calculate_size(strlen(d)));
+	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
 
 	*hmac_size = i;
 	return buf;
-	
+
 	error:
 	LDNS_FREE(d);
 	LDNS_FREE(buf);
@@ -506,12 +538,14 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_RSASHA1:
 		case LDNS_SIGN_RSASHA1_NSEC3:
 		case LDNS_SIGN_RSASHA256:
+		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
+		case LDNS_SIGN_RSASHA512_NSEC3:
 			r = RSA_generate_key((int)size, RSA_F4, NULL, NULL);
 			if (RSA_check_key(r) != 1) {
 				return NULL;
 			}
-			
+
 			ldns_key_set_rsa_key(k, r);
 			break;
 		case LDNS_SIGN_DSA:
@@ -526,10 +560,12 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 			ldns_key_set_dsa_key(k, d);
 			break;
 		case LDNS_SIGN_HMACMD5:
+		case LDNS_SIGN_HMACSHA1:
+		case LDNS_SIGN_HMACSHA256:
 			k->_key.key = NULL;
 			size = size / 8;
 			ldns_key_set_hmac_size(k, size);
-			 
+
 			hmac = LDNS_XMALLOC(unsigned char, size);
 #ifdef HAVE_SSL
 			if (RAND_bytes(hmac, size) != 1) {
@@ -549,7 +585,7 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 			}
 #endif /* HAVE_SSL */
 			ldns_key_set_hmac_key(k, hmac);
-			
+
 			ldns_key_set_flags(k, 0);
 			break;
 	}
@@ -571,7 +607,7 @@ ldns_key_print(FILE *output, const ldns_key *k)
 
 
 void
-ldns_key_set_algorithm(ldns_key *k, ldns_signing_algorithm l) 
+ldns_key_set_algorithm(ldns_key *k, ldns_signing_algorithm l)
 {
 	k->_alg = l;
 }
@@ -907,6 +943,8 @@ ldns_key2rr(const ldns_key *k)
 
 	switch (ldns_key_algorithm(k)) {
 	case LDNS_SIGN_HMACMD5:
+	case LDNS_SIGN_HMACSHA1:
+	case LDNS_SIGN_HMACSHA256:
 		ldns_rr_set_type(pubkey, LDNS_RR_TYPE_KEY);
         	break;
 	default:
@@ -915,23 +953,25 @@ ldns_key2rr(const ldns_key *k)
         }
 	/* zero-th rdf - flags */
 	ldns_rr_push_rdf(pubkey,
-			ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16, 
+			ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16,
 				ldns_key_flags(k)));
 	/* first - proto */
-	ldns_rr_push_rdf(pubkey, 
+	ldns_rr_push_rdf(pubkey,
 			ldns_native2rdf_int8(LDNS_RDF_TYPE_INT8, LDNS_DNSSEC_KEYPROTO));
-	
+
 	if (ldns_key_pubkey_owner(k)) {
 		ldns_rr_set_owner(pubkey, ldns_rdf_clone(ldns_key_pubkey_owner(k)));
 	}
-	
+
 	/* third - da algorithm */
 	switch(ldns_key_algorithm(k)) {
 		case LDNS_RSAMD5:
 		case LDNS_RSASHA1:
 		case LDNS_RSASHA1_NSEC3:
 		case LDNS_RSASHA256:
+		case LDNS_RSASHA256_NSEC3:
 		case LDNS_RSASHA512:
+		case LDNS_RSASHA512_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 						  ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
 			rsa =  ldns_key_rsa_key(k);
@@ -966,12 +1006,15 @@ ldns_key2rr(const ldns_key *k)
 			}
 			break;
 		case LDNS_SIGN_HMACMD5:
+		case LDNS_SIGN_HMACSHA1:
+		case LDNS_SIGN_HMACSHA256:
 			/* tja */
 			ldns_rr_push_rdf(pubkey,
-                                         ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_SIGN_HMACMD5));
+                                        ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG,
+						ldns_key_algorithm(k)));
                         size = ldns_key_hmac_size(k);
-                        bin = LDNS_XREALLOC(bin, unsigned char, size);
-                        memcpy(bin, ldns_key_hmac_key(k), size);
+			bin = LDNS_XREALLOC(bin, unsigned char, size);
+			memcpy(bin, ldns_key_hmac_key(k), size);
 			break;
 	}
 	/* fourth the key bin material */
@@ -1017,7 +1060,7 @@ ldns_key_deep_free(ldns_key *key)
 	}
 	if (ldns_key_hmac_key(key)) {
 		free(ldns_key_hmac_key(key));
-	}	
+	}
 	LDNS_FREE(key);
 }
 
