@@ -48,8 +48,8 @@ enum ldns_enum_rr_class
 	LDNS_RR_CLASS_CH	= 3,
 	/** Hesiod (Dyer 87) */
 	LDNS_RR_CLASS_HS	= 4,
-        /** None class, dynmaic update */
-        LDNS_RR_CLASS_NONE      = 254,
+    /** None class, dynamic update */
+    LDNS_RR_CLASS_NONE      = 254,
 	/** Any class */
 	LDNS_RR_CLASS_ANY	= 255,
 
@@ -190,6 +190,9 @@ enum ldns_enum_rr_type
 	/**  any type (wildcard) */
 	LDNS_RR_TYPE_ANY = 255,
 
+	/* RFC 4431, 5074, DNSSEC Lookaside Validation */
+	LDNS_RR_TYPE_DLV = 32769,
+
 	/* type codes from nsec3 experimental phase
 	LDNS_RR_TYPE_NSEC3 = 65324,
 	LDNS_RR_TYPE_NSEC3PARAMS = 65325, */
@@ -340,17 +343,38 @@ void ldns_rr_free(ldns_rr *rr);
 /**
  * creates an rr from a string.
  * The string should be a fully filled-in rr, like
- * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt; TYPE &lt;space&gt; RDATA.
+ * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt; 
+ * TYPE &lt;space&gt; RDATA.
+ * \param[out] n the rr to return
+ * \param[in] str the string to convert
+ * \param[in] default_ttl default ttl value for the rr.
+ *            If 0 DEF_TTL will be used
+ * \param[in] origin when the owner is relative add this. 
+ *	The caller must ldns_rdf_deep_free it.
+ * \param[out] prev the previous ownername. if this value is not NULL,
+ * the function overwrites this with the ownername found in this
+ * string. The caller must then ldns_rdf_deep_free it.
+ * \return a status msg describing an error or LDNS_STATUS_OK
+ */
+ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str,
+                                uint32_t default_ttl, ldns_rdf *origin,
+                                ldns_rdf **prev);
+
+/**
+ * creates an rr for the question section from a string, i.e.
+ * without RDATA fields
+ * Origin and previous RR functionality are the same as in
+ * ldns_rr_new_frm_str()
  * \param[out] n the rr to return
  * \param[in] str the string to convert
- * \param[in] default_ttl pointer to a default ttl for the rr. If 0 DEF_TTL will be used
  * \param[in] origin when the owner is relative add this. 
  *	The caller must ldns_rdf_deep_free it.
  * \param prev the previous ownername. the function overwrite this with
  * the current found ownername. The caller must ldns_rdf_deep_free it.
  * \return a status msg describing an error or LDNS_STATUS_OK
  */
-ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str, uint32_t default_ttl, ldns_rdf *origin, ldns_rdf **prev);
+ldns_status ldns_rr_new_question_frm_str(ldns_rr **n, const char *str,
+                                ldns_rdf *origin, ldns_rdf **prev);
 
 /**
  * creates a new rr from a file containing a string.
@@ -808,4 +832,25 @@ ldns_rr_list *ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t p
  */
 ldns_rr_type    ldns_rdf2rr_type(const ldns_rdf *rd);
 
+/**
+ * Returns the type of the first element of the RR
+ * If there are no elements present, 0 is returned
+ * (LDNS_RR_TYPE_A)
+ * 
+ * \param[in] rr_list The rr list
+ * \return rr_type of the first element, or 0 if the list is empty
+ */
+ldns_rr_type
+ldns_rr_list_type(const ldns_rr_list *rr_list);
+
+/**
+ * Returns the owner domain name rdf of the first element of the RR
+ * If there are no elements present, NULL is returned
+ * 
+ * \param[in] rr_list The rr list
+ * \return dname of the first element, or NULL if the list is empty
+ */
+ldns_rdf *
+ldns_rr_list_owner(const ldns_rr_list *rr_list);
+
 #endif /* LDNS_RR_H */
