@@ -89,13 +89,46 @@ struct mesh_area {
 	 * an empty set of super-states, thus are 'toplevel' or detached
 	 * internal opportunistic queries */
 	size_t num_detached_states;
+	/** number of reply states in the forever list */
+	size_t num_forever_states;
 
+	/** max total number of reply states to have */
+	size_t max_reply_states;
+	/** max forever number of reply states to have */
+	size_t max_forever_states;
+
+	/** stats, cumulative number of reply states jostled out */
+	size_t stats_jostled;
+	/** stats, cumulative number of incoming client msgs dropped */
+	size_t stats_dropped;
 	/** number of replies sent */
 	size_t replies_sent;
 	/** sum of waiting times for the replies */
 	struct timeval replies_sum_wait;
 	/** histogram of time values */
 	struct timehist* histogram;
+	/** (extended stats) secure replies */
+	size_t ans_secure;
+	/** (extended stats) bogus replies */
+	size_t ans_bogus;
+	/** (extended stats) rcodes in replies */
+	size_t ans_rcode[16];
+	/** (extended stats) rcode nodata in replies */
+	size_t ans_nodata;
+
+	/** double linked list of the run-to-completion query states.
+	 * These are query states with a reply */
+	struct mesh_state* forever_first;
+	/** last entry in run forever list */
+	struct mesh_state* forever_last;
+
+	/** double linked list of the query states that can be jostled out
+	 * by new queries if too old.  These are query states with a reply */
+	struct mesh_state* jostle_first;
+	/** last entry in jostle list - this is the entry that is newest */
+	struct mesh_state* jostle_last;
+	/** timeout for jostling. if age is lower, it does not get jostled. */
+	struct timeval jostle_max;
 };
 
 /**
@@ -127,6 +160,17 @@ struct mesh_state {
 	rbtree_t sub_set;
 	/** number of activations for the mesh state */
 	size_t num_activated;
+
+	/** previous in linked list for reply states */
+	struct mesh_state* prev;
+	/** next in linked list for reply states */
+	struct mesh_state* next;
+	/** if this state is in the forever list, jostle list, or neither */
+	enum mesh_list_select { mesh_no_list, mesh_forever_list, 
+		mesh_jostle_list } list_select;
+
+	/** true if replies have been sent out (at end for alignment) */
+	uint8_t replies_sent;
 };
 
 /**
@@ -248,11 +292,11 @@ int mesh_new_callback(struct mesh_area* mesh, struct query_info* qinfo,
  *
  * @param mesh: the query mesh.
  * @param e: outbound entry, with query state to run and reply pointer.
- * @param is_ok: if true, reply is OK, otherwise a timeout happened.
  * @param reply: the comm point reply info.
+ * @param what: NETEVENT_* error code (if not 0, what is wrong, TIMEOUT).
  */
 void mesh_report_reply(struct mesh_area* mesh, struct outbound_entry* e,
-	int is_ok, struct comm_reply* reply);
+	struct comm_reply* reply, int what);
 
 /* ------------------- Functions for module environment --------------- */
 
@@ -457,4 +501,29 @@ int mesh_state_compare(const void* ap, const void* bp);
 /** compare two mesh references */
 int mesh_state_ref_compare(const void* ap, const void* bp);
 
+/**
+ * Make space for another recursion state for a reply in the mesh
+ * @param mesh: mesh area
+ * @return false if no space is available.
+ */
+int mesh_make_new_space(struct mesh_area* mesh);
+
+/**
+ * Insert mesh state into a double linked list.  Inserted at end.
+ * @param m: mesh state.
+ * @param fp: pointer to the first-elem-pointer of the list.
+ * @param lp: pointer to the last-elem-pointer of the list.
+ */
+void mesh_list_insert(struct mesh_state* m, struct mesh_state** fp,
+	struct mesh_state** lp);
+
+/**
+ * Remove mesh state from a double linked list.  Remove from any position.
+ * @param m: mesh state.
+ * @param fp: pointer to the first-elem-pointer of the list.
+ * @param lp: pointer to the last-elem-pointer of the list.
+ */
+void mesh_list_remove(struct mesh_state* m, struct mesh_state** fp,
+	struct mesh_state** lp);
+
 #endif /* SERVICES_MESH_H */
