@@ -48,8 +48,12 @@
 /** The TCP reading or writing query timeout in seconds */
 #define TCP_QUERY_TIMEOUT 120 
 
+#ifndef NONBLOCKING_IS_BROKEN
 /** number of UDP reads to perform per read indication from select */
 #define NUM_UDP_PER_SELECT 100
+#else
+#define NUM_UDP_PER_SELECT 1
+#endif
 
 /* We define libevent structures here to hide the libevent stuff. */
 
@@ -209,6 +213,11 @@ void comm_base_exit(struct comm_base* b)
 	}
 }
 
+struct event_base* comm_base_internal(struct comm_base* b)
+{
+	return b->eb->base;
+}
+
 /* send a UDP reply */
 int
 comm_point_send_udp_msg(struct comm_point *c, ldns_buffer* packet,
@@ -498,7 +507,8 @@ comm_point_udp_callback(int fd, short event, void* arg)
 		if(recv == -1) {
 #ifndef USE_WINSOCK
 			if(errno != EAGAIN && errno != EINTR)
-				log_err("recvfrom failed: %s", strerror(errno));
+				log_err("recvfrom %d failed: %s", 
+					fd, strerror(errno));
 #else
 			if(WSAGetLastError() != WSAEINPROGRESS &&
 				WSAGetLastError() != WSAECONNRESET &&
@@ -517,7 +527,8 @@ comm_point_udp_callback(int fd, short event, void* arg)
 			(void)comm_point_send_udp_msg(rep.c, rep.c->buffer,
 				(struct sockaddr*)&rep.addr, rep.addrlen);
 		}
-		if(rep.c->fd == -1) /* commpoint closed */
+		if(rep.c->fd != fd) /* commpoint closed to -1 or reused for
+		another UDP port. Note rep.c cannot be reused with TCP fd. */
 			break;
 	}
 }
@@ -534,6 +545,45 @@ setup_tcp_handler(struct comm_point* c, int fd)
 	comm_point_start_listening(c, fd, TCP_QUERY_TIMEOUT);
 }
 
+int comm_point_perform_accept(struct comm_point* c,
+	struct sockaddr_storage* addr, socklen_t* addrlen)
+{
+	int new_fd;
+	*addrlen = (socklen_t)sizeof(*addr);
+	new_fd = accept(c->fd, (struct sockaddr*)addr, addrlen);
+	if(new_fd == -1) {
+#ifndef USE_WINSOCK
+		/* EINTR is signal interrupt. others are closed connection. */
+		if(	errno != EINTR 
+#ifdef EWOULDBLOCK
+			&& errno != EWOULDBLOCK 
+#endif
+#ifdef ECONNABORTED
+			&& errno != ECONNABORTED 
+#endif
+#ifdef EPROTO
+			&& errno != EPROTO
+#endif /* EPROTO */
+			)
+			return -1;
+		log_err("accept failed: %s", strerror(errno));
+#else /* USE_WINSOCK */
+		if(WSAGetLastError() == WSAEINPROGRESS ||
+			WSAGetLastError() == WSAECONNRESET)
+			return -1;
+		if(WSAGetLastError() == WSAEWOULDBLOCK) {
+			winsock_tcp_wouldblock(&c->ev->ev, EV_READ);
+			return -1;
+		}
+		log_err("accept failed: %s", wsa_strerror(WSAGetLastError()));
+#endif
+		log_addr(0, "remote address is", addr, *addrlen);
+		return -1;
+	}
+	fd_set_nonblock(new_fd);
+	return new_fd;
+}
+
 void 
 comm_point_tcp_accept_callback(int fd, short event, void* arg)
 {
@@ -552,40 +602,12 @@ comm_point_tcp_accept_callback(int fd, short event, void* arg)
 	}
 	/* accept incoming connection. */
 	c_hdl = c->tcp_free;
-	c_hdl->repinfo.addrlen = (socklen_t)sizeof(c_hdl->repinfo.addr);
 	log_assert(fd != -1);
-	new_fd = accept(fd, (struct sockaddr*)&c_hdl->repinfo.addr, 
+	new_fd = comm_point_perform_accept(c, &c_hdl->repinfo.addr,
 		&c_hdl->repinfo.addrlen);
-	if(new_fd == -1) {
-#ifndef USE_WINSOCK
-		/* EINTR is signal interrupt. others are closed connection. */
-		if(	errno != EINTR 
-#ifdef EWOULDBLOCK
-			&& errno != EWOULDBLOCK 
-#endif
-#ifdef ECONNABORTED
-			&& errno != ECONNABORTED 
-#endif
-#ifdef EPROTO
-			&& errno != EPROTO
-#endif /* EPROTO */
-			)
-			return;
-		log_err("accept failed: %s", strerror(errno));
-#else /* USE_WINSOCK */
-		if(WSAGetLastError() == WSAEINPROGRESS ||
-			WSAGetLastError() == WSAECONNRESET)
-			return;
-		if(WSAGetLastError() == WSAEWOULDBLOCK) {
-			winsock_tcp_wouldblock(&c->ev->ev, EV_READ);
-			return ;
-		}
-		log_err("accept failed: %s", wsa_strerror(WSAGetLastError()));
-#endif
-		log_addr(0, "remote address is", &c_hdl->repinfo.addr,
-			c_hdl->repinfo.addrlen);
+	if(new_fd == -1)
 		return;
-	}
+
 	/* grab the tcp handler buffers */
 	c->tcp_free = c_hdl->tcp_free;
 	if(!c->tcp_free) {
@@ -780,6 +802,10 @@ comm_point_tcp_handle_write(int fd, struct comm_point* c)
 #ifdef EHOSTUNREACH
                 else if(error == EHOSTUNREACH && verbosity < 2)
                         return 0; /* silence 'no route to host' */
+#endif
+#ifdef EHOSTDOWN
+                else if(error == EHOSTDOWN && verbosity < 2)
+                        return 0; /* silence 'host is down' */
 #endif
                 else if(error != 0) {
 			log_err("tcp connect: %s", strerror(error));
@@ -937,13 +963,17 @@ void comm_point_local_handle_callback(int fd, short event, void* arg)
 }
 
 void comm_point_raw_handle_callback(int ATTR_UNUSED(fd), 
-	short ATTR_UNUSED(event), void* arg)
+	short event, void* arg)
 {
 	struct comm_point* c = (struct comm_point*)arg;
+	int err = NETEVENT_NOERROR;
 	log_assert(c->type == comm_raw);
 	comm_base_now(c->ev->base);
-
-	(void)(*c->callback)(c, c->cb_arg, NETEVENT_NOERROR, NULL);
+	
+	if(event&EV_TIMEOUT)
+		err = NETEVENT_TIMEOUT;
+	fptr_ok(fptr_whitelist_comm_point_raw(c->callback));
+	(void)(*c->callback)(c, c->cb_arg, err, NULL);
 }
 
 struct comm_point* 
@@ -1094,7 +1124,7 @@ comm_point_create_tcp_handler(struct comm_base *base,
 	c->tcp_free = parent->tcp_free;
 	parent->tcp_free = c;
 	/* libevent stuff */
-	evbits = EV_PERSIST | EV_READ;
+	evbits = EV_PERSIST | EV_READ | EV_TIMEOUT;
 	event_set(&c->ev->ev, c->fd, evbits, comm_point_tcp_handle_callback, c);
 	if(event_base_set(base->eb->base, &c->ev->ev) != 0)
 	{
@@ -1423,6 +1453,7 @@ comm_point_start_listening(struct comm_point* c, int newfd, int sec)
 				return;
 			}
 		}
+		c->ev->ev.ev_events |= EV_TIMEOUT;
 #ifndef S_SPLINT_S /* splint fails on struct timeval. */
 		c->timeout->tv_sec = sec;
 		c->timeout->tv_usec = 0;
@@ -1445,6 +1476,20 @@ comm_point_start_listening(struct comm_point* c, int newfd, int sec)
 	}
 }
 
+void comm_point_listen_for_rw(struct comm_point* c, int rd, int wr)
+{
+	verbose(VERB_ALGO, "comm point listen_for_rw %d %d", c->fd, wr);
+	if(event_del(&c->ev->ev) != 0) {
+		log_err("event_del error to cplf");
+	}
+	c->ev->ev.ev_events &= ~(EV_READ|EV_WRITE);
+	if(rd) c->ev->ev.ev_events |= EV_READ;
+	if(wr) c->ev->ev.ev_events |= EV_WRITE;
+	if(event_add(&c->ev->ev, c->timeout) != 0) {
+		log_err("event_add failed. in cplf.");
+	}
+}
+
 size_t comm_point_get_mem(struct comm_point* c)
 {
 	size_t s;
