@@ -63,6 +63,7 @@
 #include "config.h"
 struct comm_point;
 struct comm_reply;
+struct event_base;
 
 /* internal event notification data storage structure. */
 struct internal_event;
@@ -79,6 +80,8 @@ typedef int comm_point_callback_t(struct comm_point*, void*, int,
 #define NETEVENT_CLOSED -1
 /** to pass timeout happened to callback function */
 #define NETEVENT_TIMEOUT -2 
+/** to pass fallback from capsforID to callback function; 0x20 failed */
+#define NETEVENT_CAPSFAIL -3
 
 /**
  * A communication point dispatcher. Thread specific.
@@ -289,6 +292,13 @@ void comm_base_dispatch(struct comm_base* b);
  */
 void comm_base_exit(struct comm_base* b);
 
+/**
+ * Access internal data structure (for util/tube.c on windows)
+ * @param b: comm base
+ * @return event_base. Could be libevent, or internal event handler.
+ */
+struct event_base* comm_base_internal(struct comm_base* b);
+
 /**
  * Create an UDP comm point. Calls malloc.
  * setups the structure with the parameters you provide.
@@ -435,6 +445,14 @@ void comm_point_stop_listening(struct comm_point* c);
  */
 void comm_point_start_listening(struct comm_point* c, int newfd, int sec);
 
+/**
+ * Stop listening and start listening again for reading or writing.
+ * @param c: commpoint
+ * @param rd: if true, listens for reading.
+ * @param wr: if true, listens for writing.
+ */
+void comm_point_listen_for_rw(struct comm_point* c, int rd, int wr);
+
 /**
  * Get size of memory used by comm point.
  * For TCP handlers this includes subhandlers.
@@ -512,6 +530,17 @@ int comm_signal_bind(struct comm_signal* comsig, int sig);
  */
 void comm_signal_delete(struct comm_signal* comsig);
 
+/**
+ * perform accept(2) with error checking.
+ * @param c: commpoint with accept fd.
+ * @param addr: remote end returned here.
+ * @param addrlen: length of remote end returned here.
+ * @return new fd, or -1 on error.
+ *	if -1, error message has been printed if necessary, simply drop
+ *	out of the reading handler.
+ */
+int comm_point_perform_accept(struct comm_point* c, 
+	struct sockaddr_storage* addr, socklen_t* addrlen);
 
 /**** internal routines ****/
 
